Framework BRL.Basic

Import gg.IrrBMAX
Import gg.IrrAddons

Global grass:T_irrGrassPatchSceneNode[100]

Global modes:Int[4] ' core::array<video::E_MATERIAL_TYPE>
Global tex1:T_irrITexture
Global tex2:T_irrITexture
Local wind:T_irrIWindGenerator

Type MyEventReceiver Extends T_irrIEventReceiver

	Field Terrain:T_irrISceneNode
	Field mode:Int
	Field bTex:Int
	Field Grass:T_irrGrassPatchSceneNode[]
	
	Function createGrassReceiver:T_irrIEventReceiver(terrain:T_irrISceneNode,grass:T_irrGrassPatchSceneNode[])
		Local retval:MyEventReceiver=MyEventReceiver(create(MyEventReceiver.generate))
		retval.Terrain=terrain
		retval.Grass=grass
		retval.mode=0
		Return T_irrIEventReceiver(retval)
	EndFunction

	Method OnEvent:Int(event:T_irrSEvent)

		If (event.getEventType()=EET_KEY_INPUT_EVENT And Not event.getKeyPressedDown())
			
			Local key:Int=event.getKeyInputKey()
			Local n:Int=0
			
			Select key
					
				Case EKEY_KEY_W ' switch wire frame mode
				    Terrain.setMaterialFlag(EMF_WIREFRAME, Not Terrain.getMaterial(0).getWireframe())
				    Return True
				Case EKEY_KEY_D ' toggle detail map
					If Terrain.getMaterial(0).getMaterialType()=EMT_SOLID
						Terrain.getMaterial(0).setMaterialType(EMT_DETAIL_MAP)
					Else
						Terrain.getMaterial(0).setMaterialType(EMT_SOLID)
					EndIf
				    Return True
				Case EKEY_KEY_Z ' change draw distance
				    For n=0 To 99
				        If (Grass[n].getDrawDistance() > GRASS_PATCH_SIZE + 50)
				            Grass[n].setDrawDistance(Grass[n].getDrawDistance()- 50.0)
						EndIf
				    Next
				    Return True
				Case EKEY_KEY_X ' change draw distance
				    For n=0 To 99
				        Grass[n].setDrawDistance(Grass[n].getDrawDistance()+ 50.0)
				    Next
				    Return True
				Case EKEY_KEY_C ' change Max density
				    For n=0 To 99
				          If (Grass[n].getMaxDensity() > 55 )
				            Grass[n].setMaxDensity(Grass[n].getMaxDensity()- 50)
						EndIf
				    Next
				    Return True
				Case EKEY_KEY_V ' change Max density
				    For n=0 To 99
				        Grass[n].setMaxDensity(Grass[n].getMaxDensity()+ 50)
				    Next
				    Return True
				Case EKEY_KEY_B ' change mode
				    If (mode > 0) Then mode:-1
				    For n=0 To 99
				        Grass[n].setMaterialType(modes[mode])
				    Next
				    Return True
				Case EKEY_KEY_N ' change mode
				    If (mode < 3) Then mode:+1
				    For n=0 To 99
				        Grass[n].setMaterialType(modes[mode])
				    Next
				    Return True
				Case EKEY_KEY_M ' change mode
				    bTex = Not bTex
				    For n=0 To 99
						Local tex:T_irrITexture
						If bTex Then tex=tex1 Else tex=tex2
				        Grass[n].setMaterialTexture(0,tex)
				    Next
				    Return True
				    
				Case EKEY_KEY_S ' switch terrain invisible
				    Terrain.setVisible( Not Terrain.isVisible())
				    Return True				
				Default
					Return False
			EndSelect
		EndIf

		Return False
	EndMethod

	' we must override the generate function in order for instantiation to work properly
	' must return T_irrIEventReceiver
	Function generate:T_irrIEventReceiver()
		Return T_irrIEventReceiver(New MyEventReceiver)
	EndFunction
EndType

Rem
EndRem

Rem
The start of the main Function starts like in most other example. We ask the user
For the desired renderer And start it up.
EndRem
' let user Select driver Type

Local driverType:Int=EDT_DIRECT3D9
Local strInput:String=""

strInput$= ..
	Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+ ..
	" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.2"+Chr(13)+Chr(10)+ ..
	" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Apfelbaum Software Renderer"+Chr(13)+Chr(10)+ ..
	" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select strInput$
	Case "a"
		driverType=EDT_DIRECT3D9
	Case "b"
		driverType=EDT_DIRECT3D8
	Case "c"
		driverType=EDT_OPENGL
	Case "d"
		driverType=EDT_SOFTWARE
	Case "e"
		driverType=EDT_SOFTWARE2
	Case "f"
		driverType=EDT_NULL
	Default
		Return 
EndSelect

modes[0] = EMT_TRANSPARENT_ALPHA_CHANNEL
modes[1] = EMT_TRANSPARENT_ALPHA_CHANNEL_REF
modes[2] = EMT_TRANSPARENT_ADD_COLOR
modes[3] = EMT_TRANSPARENT_VERTEX_ALPHA

' create device
Local device:T_irrIrrlichtDevice= ..
	T_irrIrrlichtDevice.create(driverType,T_irrDimension2d_s32.create(800,600))

If Not device.IsValid() Then Return ' could Not create selected driver.

Local driver:T_irrIVideoDriver=device.getVideoDriver()
Local smgr:T_irrISceneManager=device.getSceneManager()
Local env:T_irrIGUIEnvironment=device.getGUIEnvironment()

driver.setTextureCreationFlag(ETCF_ALWAYS_32_BIT, True)

' add irrlicht logo
env.addImageFromTexture(driver.getTexture("../media/irrlichtlogoalpha.tga"),T_irrPosition2d_s32.create(10,10))

' add some help text
Local text:T_irrIGUIStaticText = env.addStaticText( ..
	"Press 'S' to toggle terrain"+Chr(13)+Chr(10)+ ..
	"Press 'W' to toggle wireframe mode"+Chr(13)+Chr(10)+ ..
	"Press 'D' to toggle detail map"+Chr(13)+Chr(10)+ ..
	"Z and X for distance"+Chr(13)+Chr(10)+ ..
	"C and V for max density"+Chr(13)+Chr(10)+ ..
	"B and N to change render mode"+Chr(13)+Chr(10)+ ..
	"M to swap texture"+Chr(13)+Chr(10)+ ..
	"K and L to change wind strength"+Chr(13)+Chr(10)+ ..
	"O and P to change wind regularity", ..
	T_irrRect_s32.createFromVals(10,453,200,590), True, True, 0, -1, True)

' add camera
Local camera:T_irrICameraSceneNode = smgr.addCameraSceneNodeFPS(0,100.0,1200.0)

camera.setPosition(T_irrVector3df.createFromVals(1000,500,1000))
camera.setFarValue(12000.0)

' create wind...
wind = T_irrIWindGenerator.create( 70.0, 3.0 )

' disable mouse cursor
device.getCursorControl().setVisible(False)

' add terrain scene node
Local terrain:T_irrITerrainSceneNode = smgr.addTerrainSceneNodeFromFile("terrain-heightmap.bmp")
Local heightMap:T_irrIImage = driver.createImageFromFile ("terrain-heightmap.bmp")
Local textureMap:T_irrIImage = driver.createImageFromFile ("terrain-grasscol.bmp") 
Local ts:T_irrVector3df=T_irrVector3df.createFromVals(40, 4.0, 40)
terrain.setScale(ts)
terrain.setMaterialFlag(EMF_LIGHTING, False)

terrain.setMaterialTexture(0, driver.getTexture("terrain-texture.jpg"))
terrain.setMaterialTexture(1, driver.getTexture("detailmap3.jpg"))

terrain.setMaterialType(EMT_DETAIL_MAP)

terrain.scaleTexture(1.0, 20.0)

' create triangle selector For the terrain    
Local selector:T_irrITriangleSelector = smgr.createTerrainTriangleSelector(terrain, 0)
terrain.setTriangleSelector(selector)
selector.drop()

' create collision response animator And attach it To the camera
Local anim:T_irrISceneNodeAnimator = smgr.createCollisionResponseAnimator( ..
    selector, camera, T_irrVector3df.createFromVals(60,100,60), ..
    T_irrVector3df.createFromVals(0,0,0), T_irrVector3df.createFromVals(0,50,0))
camera.addAnimator(anim)
anim.drop()

' create event receiver

device.setEventReceiver(MyEventReceiver.createGrassReceiver(terrain,grass))

Local lastFPS:Int = -1


tex1 = driver.getTexture("grass.png")
tex2 = driver.getTexture("grass2.bmp")
driver.makeColorKeyTextureFromPixel(tex2, T_irrPosition2d_s32.create(0,0))


' now we add some grass nodes
For Local x:Int=0 To 9
	For Local z:Int=0 To 9
		' add a grass node        
		grass[x*10 + z] = T_irrGrassPatchSceneNode.create(False, smgr.getRootSceneNode(), smgr, -1, T_irrVector3di.createFromVals(x,0,z), "grass", heightMap, textureMap, ts,wind)
		grass[x*10 + z].setMaterialFlag(EMF_LIGHTING,False)
		
		grass[x*10 + z].setMaterialType(EMT_TRANSPARENT_ALPHA_CHANNEL)
		grass[x*10 + z].setMaterialTexture(0,tex1)
		grass[x*10 + z].drop()
	Next
Next

While(device.run())
	If (device.isWindowActive())
		driver.beginScene(True, True, T_irrSColor.createFromVals(0,0,0,120) )

		smgr.drawAll()
		env.drawAll()

		driver.endScene()
        
		If (camera.getPosition().getY() < -300.0)
			camera.setPosition(T_irrVector3df.createFromVals(1000,500,1000))
		EndIf
		
		' display frames per second in window title
		Local fps:Int = driver.getFPS()
		If (lastFPS <> fps)
			Local str:String = "Grass! ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			str :+ ", Tri:"
			str :+ driver.getPrimitiveCountDrawn()
			str :+ ", Draw Distance:"
			str :+ grass[0].getDrawDistance()
			str :+ ", Max density:"
			str :+ grass[0].getMaxDensity()
			
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

device.drop()
    

